import { registerProvider } from "@tsed/di";
import { DataSource } from "typeorm";
import { Logger } from "@tsed/logger";
import { User } from "../entities/User";
import { Link } from "../entities/link/Link";
import { QueryRunner } from "typeorm";

export const SqliteDatasource = Symbol.for("SqliteDatasource");
export type SqliteDatasource = DataSource;
export const sqliteDatasource =
  process.env.NODE_ENV === "test"
    ? new DataSource({
        type: "sqlite",
        entities: [User, Link],
        database: ":memory:",
        synchronize: true,
        extra: {
          connectionLimit: 1,
          busyTimeout: 4000,
          busyErrorRetry: 10
        }
      })
    : new DataSource({
        type: "sqlite",
        entities: [User, Link],
        database: "database.sqlite",
        synchronize: true,
        extra: {
          connectionLimit: 1,
          busyTimeout: 4000,
          busyErrorRetry: 10
        }
      });

registerProvider<DataSource>({
  provide: SqliteDatasource,
  type: "typeorm:datasource",
  deps: [Logger],
  async useAsyncFactory(logger: Logger) {
    await sqliteDatasource.initialize();

    logger.info("Connected with typeorm to database: Sqlite");

    return sqliteDatasource;
  },
  hooks: {
    $onDestroy(dataSource) {
      return dataSource.isInitialized && dataSource.destroy();
    }
  }
});

export async function executeWithRetry<T>(
  operation: (queryRunner: QueryRunner) => Promise<T>,
  dataSource: DataSource,
  maxRetries = 10,
  delay = 1000
): Promise<T> {
  let retries = 0;
  while (true) {
    const queryRunner = dataSource.createQueryRunner();
    try {
      await queryRunner.connect();
      await queryRunner.startTransaction();

      const result = await operation(queryRunner);

      await queryRunner.commitTransaction();
      return result;
    } catch (error) {
      await queryRunner.rollbackTransaction();

      if (error.code === "SQLITE_BUSY" && retries < maxRetries) {
        retries++;
        await new Promise((resolve) => setTimeout(resolve, delay));
        continue;
      }

      throw error;
    } finally {
      await queryRunner.release();
    }
  }
}
